@@ -14,7 +14,7 @@ from .php import PHP
 from .model import (TRUNK_COLOR, LEAF_COLOR, NODE_COLOR, GROUP_TYPE, OWNER_CONST,
                     Edge, Group, Node, Variable, is_installed, flatten)
 
-VERSION = '2.3.1'
+VERSION = '2.4.0'
 
 IMAGE_EXTENSIONS = ('png', 'svg')
 TEXT_EXTENSIONS = ('dot', 'gv', 'json')
@@ -57,6 +57,157 @@ class LanguageParams():
         self.ruby_version = ruby_version
 
 
+class SubsetParams():
+    """
+    Shallow structure to make storing subset-specific parameters cleaner.
+    """
+    def __init__(self, target_function, upstream_depth, downstream_depth):
+        self.target_function = target_function
+        self.upstream_depth = upstream_depth
+        self.downstream_depth = downstream_depth
+
+    @staticmethod
+    def generate(target_function, upstream_depth, downstream_depth):
+        """
+        :param target_function str:
+        :param upstream_depth int:
+        :param downstream_depth int:
+        :rtype: SubsetParams|Nonetype
+        """
+        if upstream_depth and not target_function:
+            raise AssertionError("--upstream-depth requires --target-function")
+
+        if downstream_depth and not target_function:
+            raise AssertionError("--downstream-depth requires --target-function")
+
+        if not target_function:
+            return None
+
+        if not (upstream_depth or downstream_depth):
+            raise AssertionError("--target-function requires --upstream-depth or --downstream-depth")
+
+        if upstream_depth < 0:
+            raise AssertionError("--upstream-depth must be >= 0. Exclude argument for complete depth.")
+
+        if downstream_depth < 0:
+            raise AssertionError("--downstream-depth must be >= 0. Exclude argument for complete depth.")
+
+        return SubsetParams(target_function, upstream_depth, downstream_depth)
+
+
+
+def _find_target_node(subset_params, all_nodes):
+    """
+    Find the node referenced by subset_params.target_function
+    :param subset_params SubsetParams:
+    :param all_nodes list[Node]:
+    :rtype: Node
+    """
+    target_nodes = []
+    for node in all_nodes:
+        if node.token == subset_params.target_function or \
+           node.token_with_ownership() == subset_params.target_function or \
+           node.name() == subset_params.target_function:
+            target_nodes.append(node)
+    if not target_nodes:
+        raise AssertionError("Could not find node %r to build a subset." % subset_params.target_function)
+    if len(target_nodes) > 1:
+        raise AssertionError("Found multiple nodes for %r: %r. Try either a `class.func` or "
+                             "`filename::class.func`." % (subset_params.target_function, target_nodes))
+    return target_nodes[0]
+
+
+def _filter_nodes_for_subset(subset_params, all_nodes, edges):
+    """
+    Given subset_params, return a set of all nodes upstream and downstream of the target node.
+    :param subset_params SubsetParams:
+    :param all_nodes list[Node]:
+    :param edges list[Edge]:
+    :rtype: set[Node]
+    """
+    target_node = _find_target_node(subset_params, all_nodes)
+    downstream_dict = collections.defaultdict(set)
+    upstream_dict = collections.defaultdict(set)
+    for edge in edges:
+        upstream_dict[edge.node1].add(edge.node0)
+        downstream_dict[edge.node0].add(edge.node1)
+
+    include_nodes = {target_node}
+    step_nodes = {target_node}
+    next_step_nodes = set()
+
+    for _ in range(subset_params.downstream_depth):
+        for node in step_nodes:
+            next_step_nodes.update(downstream_dict[node])
+        include_nodes.update(next_step_nodes)
+        step_nodes = next_step_nodes
+        next_step_nodes = set()
+
+    step_nodes = {target_node}
+    next_step_nodes = set()
+
+    for _ in range(subset_params.upstream_depth):
+        for node in step_nodes:
+            next_step_nodes.update(upstream_dict[node])
+        include_nodes.update(next_step_nodes)
+        step_nodes = next_step_nodes
+        next_step_nodes = set()
+
+    return include_nodes
+
+
+def _filter_edges_for_subset(new_nodes, edges):
+    """
+    Given the subset of nodes, filter for edges within this subset
+    :param new_nodes set[Node]:
+    :param edges list[Edge]:
+    :rtype: list[Edge]
+    """
+    new_edges = []
+    for edge in edges:
+        if edge.node0 in new_nodes and edge.node1 in new_nodes:
+            new_edges.append(edge)
+    return new_edges
+
+
+def _filter_groups_for_subset(new_nodes, file_groups):
+    """
+    Given the subset of nodes, do housekeeping and filter out for groups within this subset
+    :param new_nodes set[Node]:
+    :param file_groups list[Group]:
+    :rtype: list[Group]
+    """
+    for file_group in file_groups:
+        for node in file_group.all_nodes():
+            if node not in new_nodes:
+                node.remove_from_parent()
+
+    new_file_groups = [g for g in file_groups if g.all_nodes()]
+
+    for file_group in new_file_groups:
+        for group in file_group.all_groups():
+            if not group.all_nodes():
+                group.remove_from_parent()
+
+    return new_file_groups
+
+
+def _filter_for_subset(subset_params, all_nodes, edges, file_groups):
+    """
+    Given subset_params, return the subset of nodes, edges, and groups
+    upstream and downstream of the target node.
+    :param subset_params SubsetParams:
+    :param all_nodes list[Node]:
+    :param edges list[Edge]:
+    :param file_groups list[Group]:
+    :rtype: list[Group], list[Node], list[Edge]
+    """
+    new_nodes = _filter_nodes_for_subset(subset_params, all_nodes, edges)
+    new_edges = _filter_edges_for_subset(new_nodes, edges)
+    new_file_groups = _filter_groups_for_subset(new_nodes, file_groups)
+    return new_file_groups, list(new_nodes), new_edges
+
+
 def generate_json(nodes, edges):
     '''
     Generate a json string from nodes and edges
@@ -356,8 +507,10 @@ def map_it(sources, extension, no_trimming, exclude_namespaces, exclude_function
     # Not a step. Just log what we know so far
     logging.info("Found groups %r." % [g.label() for g in all_subgroups])
     logging.info("Found nodes %r." % sorted(n.token_with_ownership() for n in all_nodes))
-    logging.info("Found calls %r." % sorted(list(set(c.to_string() for c in flatten(n.calls for n in all_nodes)))))
-    logging.info("Found variables %r." % sorted(list(set(v.to_string() for v in flatten(n.variables for n in all_nodes)))))
+    logging.info("Found calls %r." % sorted(list(set(c.to_string() for c in
+                                                     flatten(n.calls for n in all_nodes)))))
+    logging.info("Found variables %r." % sorted(list(set(v.to_string() for v in
+                                                         flatten(n.variables for n in all_nodes)))))
 
     # 6. Find all calls between all nodes
     bad_calls = []
@@ -464,7 +617,8 @@ def _generate_graphviz(output_file, extension, final_img_filename):
             subprocess.run(command, stdout=f, check=True)
             logging.info("Graphviz finished in %.2f seconds." % (time.time() - start_time))
         except subprocess.CalledProcessError:
-            logging.warning("*** Graphviz returned non-zero exit code! Try running %r for more detail ***", ' '.join(command + ['-v', '-O']))
+            logging.warning("*** Graphviz returned non-zero exit code! "
+                            "Try running %r for more detail ***", ' '.join(command + ['-v', '-O']))
 
 
 def _generate_final_img(output_file, extension, final_img_filename, num_edges):
@@ -483,7 +637,7 @@ def _generate_final_img(output_file, extension, final_img_filename, num_edges):
 def code2flow(raw_source_paths, output_file, language=None, hide_legend=True,
               exclude_namespaces=None, exclude_functions=None,
               no_grouping=False, no_trimming=False, skip_parse_errors=False,
-              lang_params=None, level=logging.INFO):
+              lang_params=None, subset_params=None, level=logging.INFO):
     """
     Top-level function. Generate a diagram based on source code.
     Can generate either a dotfile or an image.
@@ -498,6 +652,7 @@ def code2flow(raw_source_paths, output_file, language=None, hide_legend=True,
     :param bool no_trimming: Don't trim orphaned functions / namespaces
     :param bool skip_parse_errors: If a language parser fails to parse a file, skip it
     :param lang_params LanguageParams: Object to store lang-specific params
+    :param subset_params SubsetParams: Object to store subset-specific params
     :param int level: logging level
     :rtype: None
     """
@@ -519,7 +674,8 @@ def code2flow(raw_source_paths, output_file, language=None, hide_legend=True,
     if isinstance(output_file, str):
         assert '.' in output_file, "Output filename must end in one of: %r." % set(VALID_EXTENSIONS)
         output_ext = output_file.rsplit('.', 1)[1] or ''
-        assert output_ext in VALID_EXTENSIONS, "Output filename must end in one of: %r." % set(VALID_EXTENSIONS)
+        assert output_ext in VALID_EXTENSIONS, "Output filename must end in one of: %r." % \
+                                               set(VALID_EXTENSIONS)
 
     final_img_filename = None
     if output_ext and output_ext in IMAGE_EXTENSIONS:
@@ -537,6 +693,14 @@ def code2flow(raw_source_paths, output_file, language=None, hide_legend=True,
                                            exclude_namespaces, exclude_functions,
                                            skip_parse_errors, lang_params)
 
+    if subset_params:
+        logging.info("Filtering into subset...")
+        file_groups, all_nodes, edges = _filter_for_subset(subset_params, all_nodes, edges, file_groups)
+
+    file_groups.sort()
+    all_nodes.sort()
+    edges.sort()
+
     logging.info("Generating output file...")
 
     if isinstance(output_file, str):
@@ -580,6 +744,17 @@ def main(sys_argv=None):
         '--language', choices=['py', 'js', 'rb', 'php'],
         help='process this language and ignore all other files.'
              'If omitted, use the suffix of the first source file.')
+    parser.add_argument(
+        '--target-function',
+        help='output a subset of the graph centered on this function. '
+             'Valid formats include `func`, `class.func`, and `file::class.func`. '
+             'Requires --upstream-depth and/or --downstream-depth. ')
+    parser.add_argument(
+        '--upstream-depth', type=int, default=0,
+        help='include n nodes upstream of --target-function.')
+    parser.add_argument(
+        '--downstream-depth', type=int, default=0,
+        help='include n nodes downstream of --target-function.')
     parser.add_argument(
         '--exclude-functions',
         help='exclude functions from the output. Comma delimited.')
@@ -603,7 +778,8 @@ def main(sys_argv=None):
         help='js only. Parse the source as scripts (commonJS) or modules (es6)')
     parser.add_argument(
         '--ruby-version', default='27',
-        help='ruby only. Which ruby version to parse? This is passed directly into ruby-parse. Use numbers like 25, 27, or 31.')
+        help='ruby only. Which ruby version to parse? This is passed directly into ruby-parse. '
+             'Use numbers like 25, 27, or 31.')
     parser.add_argument(
         '--quiet', '-q', action='store_true',
         help='suppress most logging')
@@ -626,6 +802,8 @@ def main(sys_argv=None):
     exclude_namespaces = list(filter(None, (args.exclude_namespaces or "").split(',')))
     exclude_functions = list(filter(None, (args.exclude_functions or "").split(',')))
     lang_params = LanguageParams(args.source_type, args.ruby_version)
+    subset_params = SubsetParams.generate(args.target_function, args.upstream_depth,
+                                          args.downstream_depth)
 
     code2flow(
         raw_source_paths=args.sources,
@@ -638,5 +816,6 @@ def main(sys_argv=None):
         no_trimming=args.no_trimming,
         skip_parse_errors=args.skip_parse_errors,
         lang_params=lang_params,
+        subset_params=subset_params,
         level=level,
     )
